# -*- coding: utf-8 -*-

from random import *
from math import *
import copy

radius = 240
pad    = 60
fpad   = 20

# offset = radius + pad
offset = 300
lbound = pad
# ubound = 2 * radius + pad
ubound = 540

class point:
    def __init__(self, x, y):
        self.x = x
        self.y = y
    def write(self):
        print "(%d, %d)" % (self.x, self.y),
    def writeln(self):
        print "(%d, %d)" % (self.x, self.y)

class vector:
    def __init__(self, x, y):
        self.x = x
        self.y = y
    def write(self):
        print "(%f, %f)" % (self.x, self.y),
    def writeln(self):
        print "(%f, %f)" % (self.x, self.y)

class TDpoint:
    def __init__(self, x, y, z):
        self.x = x
        self.y = y
        self.z = z
    def write(self):
        print "(%d, %d, %d)" % (self.x, self.y, self.z),
    def writeln(self):
        print "(%d, %d, %d)" % (self.x, self.y, self.z)

class TDvector:
    def __init__(self, x, y, z):
        self.x = x
        self.y = y
        self.z = z
    def write(self):
        print "(%f, %f, %f)" % (self.x, self.y, self.z),
    def writeln(self):
        print "(%f, %f, %f)" % (self.x, self.y, self.z)

def distance(p, q):
    return sqrt((p.x - q.x)**2 + (p.y - q.y)**2)

def TDdistance(p, q):
    return sqrt((p.x - q.x)**2 + (p.y - q.y)**2 + (p.z - q.z)**2)

def validcoordinate(x0):
    x = x0
    if x < lbound:
        x = lbound
    if x > ubound:
        x = ubound
    return round(x)

def bound(x, a):
    if x < -a:
        x = -a
    else:
        if x > a:
            x = a
    return x
        
class Graph():
    from math import *
    
    global radius
    global offset
    
    def __init__(self, n):
        self.ord = n
        self.size = 0
        self.sim = 1    # without loop -> 1
        self.wt  = 0    # weighted -> 1
        self.dir = 0    # directed -> 1
        self.bip = 0    # bipartite -> 1

        self.adj = [[0 for j in range(n)] for i in range(n)]
        self.dash = [[False for j in range(n)] for i in range(n)]
        
        if n == 1:
            self.vtx = [point(offset, offset)]
        else:
            b = []
            for i in range(n):
                w = pi * ((2.0 * i) / n - 0.5) 
                x = offset + radius * cos(w)
                y = offset + radius * sin(w)
                b.append(point(x, y))
            self.vtx = b
            
        self.AttractionConstant = 0
        self.RepulsionConstant  = 0

        # for bipartite graphs
        self.v1 = []    # list of vertices of the 1st group 
        self.v2 = []    # list of vertices of the 2nd group 

    def SetSpringMethodConstants(self):
        avgdeg = (2.0 * self.size) / self.ord
        if self.size == 0:
            self.AttractionConstant = 1
            self.RepulsionConstant = 10
        else:
            self.AttractionConstant = round( 7.0 / log(1.0 + avgdeg))
            self.RepulsionConstant  = round(50.0 / sqrt(sqrt(self.ord)))

class GraphAL:
    def __init__(self, n):
        self.ord = n
        self.size = 0
        self.sim = 1    # without loop -> 1
        self.wt  = 0    # weighted -> 1
        self.dir = 0    # directed -> 1
        self.bip = 0    # bipartite -> 1
        
        a = []
        for i in range(n):
            a.append([])
        self.adj = a

        if n == 1:
            self.vtx = [point(offset, offset)]
        else:
            b = []
            for i in range(n):
                w = pi * ((2.0 * i) / n - 0.5) 
                x = offset + radius * cos(w)
                y = offset + radius * sin(w)
                b.append(point(x, y))
            self.vtx = b
            
        self.AttractionConstant = 0
        self.RepulsionConstant  = 0

        # for bipartite graphs
        self.v1 = []    # list of vertices of the 1st group 
        self.v2 = []    # list of vertices of the 2nd group 

    def SetSpringMethodConstants(self):
        avgdeg = (2.0 * self.size) / self.ord
        if self.size == 0:
            self.AttractionConstant = 1
            self.RepulsionConstant = 10
        else:
            self.AttractionConstant = round( 7.0 / log(1.0 + avgdeg))
            self.RepulsionConstant  = round(50.0 / sqrt(sqrt(self.ord)))

"""
クラスを引数にするとアドレス参照になるので注意
"""

def SeparateVertices(g0):
    maxjump = 50
    rep = 0
    n = g0.ord
    h = GraphCopy(g0)
    
    while 1:
        ok = True
        for i in range(n):
            for j in range(i + 1, n):
                # if distance(h.vtx(i), h.vtx(j)) < 21:
                # 大きすぎるとループから脱出できないみたい
                if distance(h.vtx[i], h.vtx[j]) < 8:
                    ok = False
                    h.vtx[j].x = validcoordinate(h.vtx[j].x + (random() - 0.5) * maxjump)
                    h.vtx[j].y = validcoordinate(h.vtx[j].y + (random() - 0.5) * maxjump)

        rep += 1
        if rep == 32:
            # maxjump *= 2
            rep = 0
        if ok:
            break
    return h

# Shuffle vertices
def VertexShuffle(g0):
    h = GraphCopy(g0)
    for i in range(h.ord):
        h.vtx[i].x = pad + radius * 2 * random()
        h.vtx[i].y = pad + radius * 2 * random()
    h = SeparateVertices(h)
    return h

# Magnetic Spring Method
def SpringMethod(g0):
    n = g0.ord
    repeat = 0
    repeatmax = 32
    g = g0
    h = Graph(n)

    d0 = 512.0 / sqrt(n)
    avgdeg = g0.size * 2.0 / n
    if g0.size > 0:
        ca = g0.AttractionConstant / (20.0 * avgdeg)
    else:
        ca = 1
    cr = g0.RepulsionConstant  / 10

    while 1:
        m = 0.0
        repeat += 1
        h = g
        for i in range(n):
            force = vector(0.0, 0.0)
            x = g.vtx[i].x
            y = g.vtx[i].y
            for j in range(n):
                if i != j:
                    direction = vector(g.vtx[j].x - x, g.vtx[j].y - y)
                    d = distance(g.vtx[j], g.vtx[i])
                    direction.x /= d
                    direction.y /= d

                    dr = d0 / d
                    # repulsion
                    if dr > 0:
                        #f = -cr * dr
                        f = -cr * (dr ** 2)

                    # attraction
                    if g.adj[i][j] > 0 or g.adj[j][i] > 0:
                        #f += ca * log(d / d0) 
                        f += ca * (d - d0)

                    force.x += f * direction.x
                    force.y += f * direction.y
                    
            force.x = bound(force.x, 50)
            force.y = bound(force.y, 50)
            h.vtx[i].x = validcoordinate(h.vtx[i].x + force.x)
            h.vtx[i].y = validcoordinate(h.vtx[i].y + force.y)
            m += (abs(h.vtx[i].x - x) + abs(h.vtx[i].y - y))

        g = SeparateVertices(h)
        if m <= 3.0 or repeat >= repeatmax:
            break
    return g

# 3-Dimensional Magnetic Spring Method
def TDSpringMethod(g0):
    n = g0.ord
    repeat = 0
    repeatmax = 32
    h = g0
    g = g0 

    d0 = 512.0 / sqrt(n)
    avgdeg = g0.size * 2.0 / n
    if g0.size > 0:
        ca = g0.AttractionConstant / (20.0 * avgdeg)
    else:
        ca = 1
    cr = g0.RepulsionConstant / 10

    TDvtx = []
    for i in range(n):
        TDvtx.append(TDpoint(h.vtx[i].x, h.vtx[i].y, 480.0 * random()))

    while 1:
        m = 0.0
        repeat += 1
        TDvtx0 = []
        for i in range(n):
            TDvtx0.append(TDvtx[i])
        for i in range(n):
            force = TDvector(0.0, 0.0, 0.0)
            x = TDvtx0[i].x
            y = TDvtx0[i].y
            z = TDvtx0[i].z
            for j in range(n):
                if i != j:
                    direction = TDvector(TDvtx0[j].x - x, TDvtx0[j].y - y, TDvtx0[j].z - z)
                    d = TDdistance(TDvtx0[j], TDvtx0[i])
                    direction.x /= d
                    direction.y /= d
                    direction.z /= d

                    dr = d0 / d
                    # repulsion
                    if dr > 0:
                        #f = -cr * dr
                        f = -cr * (dr ** 2)

                    # attraction
                    if g.adj[i][j] > 0 or g.adj[j][i] > 0:
                        #f += ca * Math.Log(d / d0) 
                        f += ca * (d - d0)

                    force.x += f * direction.x
                    force.y += f * direction.y
                    force.z += f * direction.z

            force.x = bound(force.x, 50)
            force.y = bound(force.y, 50)
            force.z = bound(force.z, 50)
            TDvtx[i].x = validcoordinate(TDvtx[i].x + force.x)
            TDvtx[i].y = validcoordinate(TDvtx[i].y + force.y)
            TDvtx[i].z = validcoordinate(TDvtx[i].z + force.z)
            m += (abs(TDvtx[i].x - x) + abs(TDvtx[i].y - y) + abs(TDvtx[i].z - z))

        for i in range(n):
            h.vtx[i].x = TDvtx[i].x
            h.vtx[i].y = TDvtx[i].y

        g = SeparateVertices(h)
        for i in range(n):
            TDvtx[i].x = g.vtx[i].x
            TDvtx[i].y = g.vtx[i].y

        if m <= 3.0 or repeat >= repeatmax:
            break
    return g

########## リスト操作関数 ##########

# リストのコピー
def ListCopy(s):
    t = []
    for x in s:
        t.append(x)
    return t

# 空リスト
# []

# リスト s 内に k が存在するときは 1、そうでないときは 0 を返す
def ListMemberQ(k, s):
    if s.count(k) > 0:
        return 1
    else:
        return 0

# リスト s 内に k が存在するときはその（最初の）ポジション、
# そうでないときは -1 を返す
def ListPosition(k, s):
    if s.count(k) > 0:
        return s.index(k)
    else:
        return -1

# リスト s の最後に k を付け加えたリスト
# s.append(k)

# リスト s の最初に k を付け加えたリスト
def Prepend(k, s):
    return s.insert(0, k)

# リスト s 内から k を全て削除したリスト
def ListRemove(k, s):
    t = []
    for x in s:
        if x != k:
            t.append(x)
    return t

# リスト s の後ろにリスト t を結合したリスト
# List ListAdd(s, t)
# s + t

# 長さ n、全てのメンバー v のリスト
# [v] * n

# リスト s の i 番目から j 番目を抜き出したリスト
# s[i: j + 1]

# リスト s の i 番目から j 番目を t に置き換えたリストを返す 
# s[0: i + 1] + t + s[j:]

# リスト s の要素を昇順にソートしたリスト
# s.sort()

# リスト s の要素を逆に並べたリスト
# s.reverse()

# リスト s, t の共通メンバーのリスト
def ListCommon(s, t):
    u = []
    for x in s:
        if x in t:
            u.append(x)
    return u

# リスト s, t の合併リスト
def ListUnion(s, t):
    u = ListCopy(s)
    for x in t:
        if not x in u:
            u.append(x)
    return u

# リスト s の全てのメンバーが t に含まれるか否か判定する
def ListIncludedQ(s, t):
    for x in s:
        if not x in t:
            return False
    return True

########## グラフ入出力関数定義部 ##########
"""
/* 標準入力からグラフを読み込む
/* 最初に頂点数、続いて順に g.adj[0][0], g.adj[0][1],...
Graph ReadGraph(void)
{
    int ord,i,j;
    Graph g;
    scanf("%d",&ord);
    g.ord=ord;
    for(i=0;i<ord;i++) for(j=0;j<ord;j++) scanf("%d",&(g.adj[i][j]));
    return g;
}

/* ２次元配列をグラフに読み替える
Graph ArraytoGraph(int n, int (*m)[n])
{
    Graph g;
    int i,j;
    g.ord=n;
    for(i=0;i<n;i++) for(j=0;j<n;j++) g.adj[i][j]=m[i][j];
    return g;
}
"""

def GraphCopy(g):
    h = copy.deepcopy(g)
    return h
    

# 隣接行列 -> 隣接リスト
def AMtoAL(g):
    h = GraphCopy(g)
    b = []
    n = g.ord
    for i in range(n):
        x = []
        for j in range(n):
            if g.adj[i][j]:
                x.append(j)
        b.append(x)
    h.adj = b
    return h

# 隣接リスト -> 隣接行列
def ALtoAM(g):
    h = GraphCopy(g)
    b = []
    n = g.ord
    for i in range(n):
        x = []
        for j in range(n):
            if j in g.adj[i]:
                x.append(1)
            else:
                x.append(0)
        b.append(x)
    h.adj = b
    return h

# グラフの画面表示
def WriteGraph(g):
    p = g.ord
    print "order =", p
    print "adjacency matrix:"
    if g.wt == 0:
        for i in range(p):
            for j in range(p):
                print g.adj[i][j],
            print
    else:
        for i in range(p):
            for j in range(p):
                print "%2d" % g.adj[i][j],
            print
        
def WriteGraphAL(g):
    p = g.ord
    print "order =", p
    print "adjacency list:"
    for i in range(p):
        for x in g.adj[i]:
            print x,
        print
        
"""
/* 重み付きグラフの隣接行列を画面表示する
void WriteWeightedGraph(Graph g)
{
    int i,j;
    for(i=0;i<g.ord;i++){
        for(j=0;j<g.ord;j++) printf("%4d",g.adj[i][j]);
        printf("\n");
    }
}
"""

########## グラフ演算関数 ##########

# 頂点数 p の空グラフ
def EmptyGraph(p):
    return Graph(p)

def EmptyGraphAL(p):
    return GraphAL(p)

# グラフ g が辺 uv を持つかどうかの判定
def EdgeQ(g, u, v):
    return (g.adj[u][v] > 0)

def EdgeQAL(g, u, v):
    return (v in g.adj[u])

# 無向グラフ g の頂点 u の次数
def VertexDegree(g, u):
    x = 0
    for i in range(g.ord):
        x += g.adj[u][i]
    return x

def VertexDegreeAL(g, u):
    return len(g.adj[u])

# 有向グラフ g の頂点 u の出次数 
def VertexOutDegree(g, u):
    x = 0
    for i in range(g.ord):
        x += g.adj[u][i]
    return x

def VertexOutDegreeAL(g, u):
    return len(g.adj[u])

# 有向グラフ g の頂点 u の入次数
def VertexInDegree(g, u):
    x = 0
    for i in range(g.ord):
        x += g.adj[i][u]
    return x

def VertexInDegreeAL(g, u):
    x = 0
    for i in range(g.ord):
        if u in g.adj[i]:
            x += 1
    return x

# 無向グラフ g の次数のリスト
def VertexDegreeList(g):
    s = []
    for i in range(g.ord):
        s.append(VertexDegree(g, i))
    return s

# 無向グラフ g の辺数
# 多重辺は数えない
# ループがあっても可
def CountEdges(g):
    x = 0
    for i in range(g.ord):
        for j in range(i, g.ord):
            if g.adj[i][j] > 0:
                x += 1
    return x

def CountEdgesAL(g):
    x = 0
    for i in range(g.ord):
        x += len(g.adj[i])
    return x / 2

# 有向グラフ g の弧数
# 多重弧は数えない
# ループがあっても可
def CountArcs(g):
    x = 0
    for i in range(g.ord):
        for j in range(g.ord):
            x += g.adj[i][j]
    return x

# Graph g が空グラフであるか否かを判定する
def EmptyGraphQ(g):
    return (g.size == 0)

# グラフ g の辺 uv を除去したグラフを返す
# 多重辺の場合は1本のみ除去
def RemoveEdge(g, u, v):
    h = GraphCopy(g)
    h.adj[u][v] -= 1
    h.adj[v][u] -= 1
    h.size -= 1
    return h

def RemoveEdgeAL(g, u, v):
    h = GraphCopy(g)
    (h.adj[u]).remove(v)
    (h.adj[v]).remove(u)
    h.size -= 1
    return h

# グラフ g からグラフ h の辺を除去したグラフを返す
# 多重辺・重み付きの場合は差を計算
def RemoveEdges(g, h):
    k = GraphCopy(g)
    p = g.ord
    for i in range(p):
        for j in range(p):
            k.adj[i][j] -= h.adj[i][j]
    k.size = g.size - h.size
    return k

def RemoveEdgesAL(g, h):
    k = GraphCopy(g)
    p = g.ord
    for i in range(p):
        for j in h.adj[i]:
            k.adj[i].remove(j)
    k.size = g.size - h.size
    return k

# グラフ g に辺 uv を付加したグラフを返す
def AddEdge(g, u, v):
    h = GraphCopy(g)
    h.adj[u][v] += 1
    h.adj[v][u] += 1
    h.size += 1
    return h

def AddEdgeAL(g, u, v):
    h = GraphCopy(g)
    (h.adj[u]).append(v)
    (h.adj[v]).append(u)
    (h.adj[u]).sort()
    (h.adj[v]).sort()
    h.size += 1
    return h

# グラフ g の点 u を除去したグラフ
def RemoveVertex(g, u):
    p = g.ord - 1
    h = Graph(p)
    for i in range(u):
        for j in range(i, u):
            h.adj[i][j] = g.adj[i][j]
            h.adj[j][i] = g.adj[j][i]
        for j in range(u, p):
            h.adj[i][j] = g.adj[i][j + 1]
            h.adj[j][i] = g.adj[j + 1][i]
    for i in range(u, p):
        for j in range(i, p):
            h.adj[i][j] = g.adj[i + 1][j + 1]
            h.adj[j][i] = g.adj[j + 1][i + 1]
    h.size = CountEdges(h)
    return h

def RemoveVertexAL(g, u):
    p = g.ord - 1
    h = GraphAL(p)
    for i in range(g.ord):
        if i != u:
            x = []
            for j in g.adj[i]:
                if j < u:
                    x.append(j)
                if j > u:
                    x.append(j - 1)
            (h.adj[i]).append(x)
    h.size = CountEdgesAL(h)
    return h

# グラフ g に点をひとつ付加したグラフ
# 第２引数は新しい点の隣接点のリスト
def AddVertex(g, nbd):
    p = g.ord 
    h = Graph(p + 1)
    for i in range(p):
        for j in range(p):
            h.adj[i][j] = g.adj[i][j]
            h.adj[j][i] = g.adj[j][i]
    for i in nbd:
        h.adj[i][p] = 1
        h.adj[p][i] = 1
    h.size = g.size + len(nbd)
    return h

# 有向グラフ g に点をひとつ付加したグラフ
# 第２引数は新しい点から隣接する点のリスト
# 第３引数は新しい点へ隣接する点のリスト
def AddVertexDirected(g, nbd1, nbd2):
    p = g.ord 
    h = Graph(p + 1)
    for i in range(p):
        for j in range(p):
            h.adj[i][j] = g.adj[i][j]
            h.adj[j][i] = g.adj[j][i]
    for i in nbd1:
        h.adj[p][i] = 1
    for i in nbd2:
        h.adj[i][p] = 1
    h.size = g.size + len(nbd1) + len(nbd2)
    h.dir = 1
    return h

# グラフ g の辺 uv を縮約したグラフ
def ContractEdge(g, u, v):
    h = GraphCopy(g)
    if u > v:
        u, v = v, u
    for i in range(g.ord):
        if i != u and i != v:
            h.adj[u][i] = g.adj[u][i] + g.adj[v][i]
            h.adj[i][u] = h.adj[u][i]
    h.adj[u][u] = 0
    h = RemoveVertex(h, v)
    #h.SetSpringMethodConstants()
    return h

def ContractEdgeAL(g, u, v):
    h = GraphCopy(g)
    if u > v:
        u, v = v, u
    for i in range(h.ord):
        while v in h.adj[i]:
            h.adj[i].append(u)
            h.adj[i].remove(v)
        h.adj[i].sort()
    for i in h.adj[v]:
        h.adj[u].append(i)
    (h.adj[u]).sort()
    h = RemoveVertexAL(h, v)
    h.SetSpringMethodConstants()
    return h

# グラフ g, h の和
def GraphSum(g, h):
    p = g.ord
    q = h.ord
    k = Graph(p + q)
    
    for i in range(p):
        for j in range(i, p):
            k.adj[i][j] = g.adj[i][j]
            k.adj[j][i] = g.adj[j][i]
            k.dash[i][j] = g.dash[i][j]
            k.dash[j][i] = g.dash[j][i]
    for i in range(q):
        for j in range(i, q):
            k.adj[p + i][p + j] = h.adj[i][j]
            k.adj[p + j][p + i] = h.adj[j][i]
            k.dash[p + i][p + j] = h.dash[i][j]
            k.dash[p + j][p + i] = h.dash[j][i]

    for i in range(p):
        k.vtx[i].x = (g.vtx[i].x - pad) / 2 + pad
        k.vtx[i].y = (g.vtx[i].y - pad) / 2 + pad
    for i in range(q):
        k.vtx[p + i].x = (h.vtx[i].x - pad) / 2 + radius + pad
        k.vtx[p + i].y = (h.vtx[i].y - pad) / 2 + radius + pad

    k.sim = g.sim * g.sim
    k.wt = g.wt + h.wt - g.wt * h.wt
    k.dir = g.dir + h.dir - g.dir * h.dir
    k.size = g.size + h.size
    k.SetSpringMethodConstants()
    return k

########## グラフ生成関数 ##########

# 頂点数 n の完全グラフ
def CompleteGraph(n):
    g = Graph(n)
    for i in range(n):
        for j in range(n):
            g.adj[i][j] = 1
    for i in range(n):
        g.adj[i][i] = 0
    g.size = (n * (n - 1)) / 2
    g.SetSpringMethodConstants()
    return g

def CompleteGraphAL(n):
    g = GraphAl(n)
    for i in range(n):
        x = range(i) + range(i + 1, n)
        g.append(x)
    g.size = (n * (n - 1)) / 2
    g.SetSpringMethodConstants()
    return g

# 完全二部グラフ K_{m,n}
def CompleteBipartiteGraph(m, n):
    g = Graph(m + n)
    for i in range(m):
        for j in range(m, m + n):
            g.adj[i][j] = 1
            g.adj[j][i] = 1
    if m == 1:
        g.vtx[0].x = offset
        g.vtx[0].y = offset - radius * 0.6
    else:
        for i in range(m):
            g.vtx[i].x = offset + ((2 * i - m + 1.0) * radius) / (m - 1) * 0.8
            g.vtx[i].y = offset - radius * 0.6
    if n == 1:
        g.vtx(m).x = offset
        g.vtx(m).y = offset + radius * 0.6
    else:
        for j in range(n):
            g.vtx[m + j].x = offset + ((2 * j - n + 1.0) * radius) / (n - 1) * 0.8
            g.vtx[m + j].y = offset + radius * 0.6
    g.size = m * n
    g.SetSpringMethodConstants()
    g.bip = 1
    g.v1 = range(m)
    g.v2 = range(m, m + n)
    return g

def CompleteBipartiteGraphAL(m, n):
    g = GraphAL(m + n)
    for i in range(m):
        g.adj[i].append(range(m, m + n))
    for i in range(m, m + n):
        g.adj[i].append(range(m))
    if m == 1:
        g.vtx[0].x = offset
        g.vtx[0].y = offset - radius * 0.6
    else:
        for i in range(m):
            g.vtx[i].x = offset + ((2 * i - m + 1.0) * radius) / (m - 1) * 0.8
            g.vtx[i].y = offset - radius * 0.6
    if n == 1:
        g.vtx(m).x = offset
        g.vtx(m).y = offset + radius * 0.6
    else:
        for j in range(n):
            g.vtx[m + j].x = offset + ((2 * j - n + 1.0) * radius) / (n - 1) * 0.8
            g.vtx[m + j].y = offset + radius * 0.6
    g.size = m * n
    g.SetSpringMethodConstants()
    g.bip = 1
    g.v1 = range(m)
    g.v2 = range(m, m + n)
    return g

# 頂点数 n の閉路
def Cycle(n):
    g = Graph(n)
    for i in range(n):
        g.adj[i][(i + 1) % n] = 1
        g.adj[(i + 1) % n][i] = 1
    g.size = n
    g.SetSpringMethodConstants()
    return g

def CycleAL(n):
    g = GraphAL(n)
    for i in range(n):
        g.adj[i].append((i - 1) % n)
        g.adj[i].append((i + 1) % n)
        g.adj[i].sort()
    g.size = n
    g.SetSpringMethodConstants()
    return g

# 頂点数 n の星グラフ
def StarGraph(n):
    g = Graph(n)
    for i in range(n - 1):
        g.adj[n - 1][i] = 1
        g.adj[i][n - 1] = 1
    g.vtx[n - 1].x = offset
    g.vtx[n - 1].y = offset
    for i in range(n - 1):
        w = pi * ((2.0 * i) / (n - 1) - 0.5) 
        g.vtx[i].x = offset + radius * cos(w)
        g.vtx[i].y = offset + radius * sin(w)
    g.size = n - 1
    g.SetSpringMethodConstants()
    return g

# 頂点数 n の車輪グラフ
def WheelGraph(n):
    g = StarGraph(n)
    for i in range(n - 1):
        j = (i + 1) % (n - 1)
        g.adj[i][j] = 1
        g.adj[j][i] = 1
    g.size = 2 * n - 2
    g.SetSpringMethodConstants()
    return g

# k-次元立方体グラフ Qk
def kDimensionalCubicGraph(k):
    if k == 0:
        return Graph(1)
    else:
        g = kDimensionalCubicGraph(k - 1)
        s = g.ord
    h = GraphSum(g, g)
    for i in range(s):
            h.adj[i][s + i] = 1
            h.adj[s + i][i] = 1
    h.SetSpringMethodConstants()
    return h

# ピータースングラフ
def PetersenGraph():
    g = EmptyGraph(10)
    for i in range(5):
        g.adj[i][(i + 1) % 5] = 1
        g.adj[(i + 1) % 5][i] = 1
    for i in range(5):
        g.adj[5 + i][5 + (i + 2) % 5] = 1
        g.adj[5 + (i + 2) % 5][5 + i] = 1
    for i in range(5):
        g.adj[i][5 + i] = 1
        g.adj[5 + i][i] = 1
    for i in range(5):
        w = pi * ((2.0 * i) / 5 - 0.5) 
        g.vtx[i].x = offset + radius * cos(w)
        g.vtx[i].y = offset + radius * sin(w)
        g.vtx[5 + i].x = offset + radius * cos(w) / 2
        g.vtx[5 + i].y = offset + radius * sin(w) / 2
    g.size = 15
    g.SetSpringMethodConstants()
    return g

# 正四面体グラフ
def PlatonicGraph4():
    g = CompleteGraph(4)
    g.vtx[3].x = offset
    g.vtx[3].y = offset + 40
    for i in range(3):
        w = pi * ((2.0 * i) / 3 - 0.5) 
        g.vtx[i].x = offset + radius * cos(w)
        g.vtx[i].y = offset + radius * sin(w) + 40
    g.size = 6
    g.SetSpringMethodConstants()
    return g

# 立方体グラフ
def PlatonicGraph6():
    g = Graph(8)
    for i in range(4):
        j = (i + 1) % 4
        g.adj[i][j] = 1
        g.adj[j][i] = 1
        g.adj[i + 4][j + 4] = 1
        g.adj[j + 4][i + 4] = 1
    for i in range(4):
        g.adj[i][i + 4] = 1
        g.adj[i + 4][i] = 1
    for i in range(4):
        w = pi * ((2.0 * i) / 4 - 0.25) 
        g.vtx[i].x = offset + radius * cos(w)
        g.vtx[i].y = offset + radius * sin(w)
        g.vtx[4 + i].x = offset + radius * cos(w) / 2
        g.vtx[4 + i].y = offset + radius * sin(w) / 2
    g.size = 12
    g.AttractionConstant = 15
    g.RepulsionConstant = 40
    return g

# 正八面体グラフ
def PlatonicGraph8():
    g = Graph(6)
    for i in range(6):
        for j in range(6):
            g.adj[i][j] = 1
    for i in range(3):
        g.adj[i][i + 3] = 0
        g.adj[i + 3][i] = 0
    for i in range(3):
        w = pi * ((2.0 * i) / 3 - 0.5) 
        g.vtx[i].x = offset + radius * cos(w)
        g.vtx[i].y = offset + radius * sin(w) + 40
    for i in range(3, 6):
        w = pi * ((2.0 * i) / 3 - 1.5) 
        g.vtx[i].x = offset + radius * cos(w) / 4
        g.vtx[i].y = offset + radius * sin(w) / 4 + 40
    g.size = 12
    g.AttractionConstant = 15
    g.RepulsionConstant = 40
    return g

# 正十二面体グラフを返す
def PlatonicGraph12():
    g = Graph(20)
    for i in range(5):
        j = (i + 1) % 5
        g.adj[i][j] = 1
        g.adj[j][i] = 1
        g.adj[i + 15][j + 15] = 1
        g.adj[j + 15][i + 15] = 1
        g.adj[i][i + 5] = 1
        g.adj[i + 5][i] = 1
        g.adj[i + 10][i + 15] = 1
        g.adj[i + 15][i + 10] = 1
    for i in range(5, 10):
        j = 10 + (i + 7) % 5
        k = 10 + (i + 8) % 5
        g.adj[i][j] = 1
        g.adj[j][i] = 1
        g.adj[i][k] = 1
        g.adj[k][i] = 1
    for i in range(5):
        w = pi * ((2.0 * i) / 5 - 0.5) 
        g.vtx[i].x = offset + radius * cos(w)
        g.vtx[i].y = offset + radius * sin(w)
    for i in range(5, 10):
        w = pi * ((2.0 * i) / 5 - 0.5) 
        g.vtx[i].x = offset + radius * cos(w) * 0.8
        g.vtx[i].y = offset + radius * sin(w) * 0.8
    for i in range(10, 15):
        w = pi * ((2.0 * i) / 5 - 1.5) 
        g.vtx[i].x = offset + radius * cos(w) * 0.55
        g.vtx[i].y = offset + radius * sin(w) * 0.55
    for i in range(15, 20):
        w = pi * ((2.0 * i) / 5 - 1.5) 
        g.vtx[i].x = offset + radius * cos(w) * 0.3
        g.vtx[i].y = offset + radius * sin(w) * 0.3
    g.size = 30
    g.AttractionConstant = 25
    g.RepulsionConstant = 40
    return g

def PlatonicGraph20():
    g = Graph(12)
    g.adj = [\
        [0,1,1,0,1,1,1,0,0,0,0,0],\
        [1,0,1,1,0,1,0,1,0,0,0,0],\
        [1,1,0,1,1,0,0,0,1,0,0,0],\
        [0,1,1,0,0,0,0,1,1,1,0,0],\
        [1,0,1,0,0,0,1,0,1,0,1,0],\
        [1,1,0,0,0,0,1,1,0,0,0,1],\
        [1,0,0,0,1,1,0,0,0,0,1,1],\
        [0,1,0,1,0,1,0,0,0,1,0,1],\
        [0,0,1,1,1,0,0,0,0,1,1,0],\
        [0,0,0,1,0,0,0,1,1,0,1,1],\
        [0,0,0,0,1,0,1,0,1,1,0,1],\
        [0,0,0,0,0,1,1,1,0,1,1,0]]
    for i in range(3):
        w = pi * ((2.0 * i) / 3 - 0.5) 
        g.vtx[i].x = offset + radius * cos(w)
        g.vtx[i].y = offset + radius * sin(w) + 40
    for i in range(3, 6):
        w = pi * ((2.0 * i) / 3 - 1.5) 
        g.vtx[i].x = offset + radius * cos(w) * 0.35
        g.vtx[i].y = offset + radius * sin(w) * 0.35 + 40
    for i in range(6, 9):
        w = pi * ((2.0 * i) / 3 - 0.5) 
        g.vtx[i].x = offset + radius * cos(w) * 0.35
        g.vtx[i].y = offset + radius * sin(w) * 0.35 + 40
    for i in range(9, 12):
        w = pi * ((2.0 * i) / 3 - 1.5) 
        g.vtx[i].x = offset + radius * cos(w) * 0.2
        g.vtx[i].y = offset + radius * sin(w) * 0.2 + 40
    g.size = 30
    g.AttractionConstant = 20
    g.RepulsionConstant = 55
    return g

# ランダムな単純無向グラフ
# 頂点数 = n, 辺の割合 = rate
# （頂点数が大きいと次数がほとんど揃ってしまう）
def RandomGraph(n, rate):
    g = Graph(n)
    k = 0
    for i in range(n):
        for j in range(i + 1, n):
            if random() <= rate:
                g.adj[i][j] = 1
                g.adj[j][i] = 1
                k += 1
    g.size = k
    g.SetSpringMethodConstants()
    return g

def RandomGraphAL(n, rate):
    g = Graph(n)
    k = 0
    for i in range(n):
        x = []
        if i > 0:
            for j in range(i):
                if i in g.adj[j]:
                    x.append(j)
        for j in range(i + 1, n):
            if random() <= rate:
                x.append(j)
                k += 1
        g.append(x)
    g.size = k
    g.SetSpringMethodConstants()
    return g

# ランダムな単純無向グラフ
# 頂点数 = n, 辺数 = m
def RandomGraphOfGivenSize(n, m):
    if m < 0:
        m = 0
    full = (n * (n - 1) / 2)
    if m > full:
        m = full
    g = Graph(n)
    k = 0
    n1 = n - 1
    while k < m:
        i = randint(0, n1)
        j = randint(0, n1)
        if i != j:
            if g.adj[i][j] == 0: 
                g.adj[i][j] = 1
                g.adj[j][i] = 1
                k += 1
    g.size = m
    g.SetSpringMethodConstants()
    return g

# ランダムな二部グラフ
# 頂点数 = m, n, 辺の割合 = rate
def RandomBipartiteGraph(m, n, rate):
    g = Graph(m + n)
    for i in range(m):
        for j in range(m, m + n):
            if random() <= rate:
                g.adj[i][j] = 1
                g.adj[j][i] = 1
    if m == 1:
        g.vtx[0].x = offset
        g.vtx[0].y = offset - radius * 0.6
    else:
        for i in range(m):
            g.vtx[i].x = offset + ((2 * i - m + 1.0) * radius) / (m - 1) * 0.8
            g.vtx[i].y = offset - radius * 0.6
    if n == 1:
        g.vtx(m).x = offset
        g.vtx(m).y = offset + radius * 0.6
    else:
        for j in range(n):
            g.vtx[m + j].x = offset + ((2 * j - n + 1.0) * radius) / (n - 1) * 0.8
            g.vtx[m + j].y = offset + radius * 0.6
    g.size = CountEdges(g)
    g.SetSpringMethodConstants()
    g.bip = 1
    g.v1 = range(m)
    g.v2 = range(m, m + n)
    return g

# ランダムな重み付き二部グラフ
# 頂点数 = m, n, 辺の割合 = rate, 最大重み = mx
def RandomWeightedBipartiteGraph(m, n, rate, mx):
    g = RandomBipartiteGraph(m, n, rate)
    g.wt = 1
    for i in range(m):
        for j in range(m, m + n):
            if g.adj[i][j] > 0:
                w = randint(1, mx)
                g.adj[i][j] = w
                g.adj[j][i] = w
    return g

# ランダムな単純無向重みつきグラフ
# 頂点数 = n, 辺の割合 = rate, 最大重み = m
def RandomWeightedGraph(n, rate, m):
    g = EmptyGraph(n)
    for i in range(n):
        for j in range(i + 1, n):
            if random() <= rate:
                w = randint(1, m)
                g.adj[i][j] = w
                g.adj[j][i] = w
    g.size = CountEdges(g)
    g.SetSpringMethodConstants()
    return g

########## グラフ探索関数 ##########

# グラフ g を根 r から深さ優先探索する
# 返り値は深さ優先探索木 t と親のリスト pl 
def DFSTree(g, r):
    # t: 深さ優先探索木
    # pl: 親のリスト
    # cl: 探索が終わったか否かを登録するリスト ( closed list )
    # k: 探索済みの頂点の個数
    # c: 現在の探索点 ( current node )
    # b: 探索木をバックトラック ( backtrack ) するか否か
    # f: 連結成分を探索し終わったか否か ( finish )

    n = g.ord               # 頂点数
    c = r                   # 探索点の初期値は根 r
    t = Graph(n)            # 木の初期化
    pl = [-1] * n           # 親が居ないと -1 を登録
    cl = [False] * n
    cl[r] = True            # 根 r は探索済み
    k = 1

    while True:
        b = True
        f = False
        for i in range(n):
            # 現在の探索点の隣接点で未探索の点を探し見つかれば
            if i != c and g.adj[c][i] > 0 and not cl[i]:
                pl[i] = c       # 親を登録
                t.adj[i][c] = 1     # 探索木の辺を登録
                t.adj[c][i] = 1 
                cl[i] = True    # 探索済みリストに登録
                k += 1
                c = i           # 探索点を i に進める
                b = False       # バックトラックは不要
                break
        # バックトラックが必要なとき
        if b:
            # 現在点が r ならこれ以上の探索は不可能
            # そうでなければ探索点の親へ戻る
            if c == r:
                f = True   
            else:
                c = pl[c]
                
        # 全ての点を探索し終わったか、探索続行不可能で終了
        if k == n or f:
            break
        
    for i in range(n):
        t.vtx[i] = g.vtx[i]
    t.size = k - 1
    t.SetSpringMethodConstants()    
    return [t, pl]

# 隣接リスト version
def DFSTreeAL(g, r):
    n = g.ord               # 頂点数
    c = r                   # 探索点の初期値は根 r
    t = GraphAL(n)          # 木の初期化
    pl = [-1] * n           # 親が居ないと -1 を登録
    cl = [False] * n
    cl[r] = True            # 根 r は探索済み
    k = 1

    while True:
        b = True
        f = False
        for i in g.adj[c]:
            # 現在の探索点の隣接点で未探索の点を探し見つかれば
            if i != c and not cl[i]:
                pl[i] = c       # 親を登録
                t.adj[i].append(c)  # 探索木の辺を登録
                t.adj[c].append(i)
                cl[i] = True    # 探索済みリストに登録
                k += 1
                c = i           # 探索点を i に進める
                b = False       # バックトラックは不要
                break
        # バックトラックが必要なとき
        if b:
            # 現在点が r ならこれ以上の探索は不可能
            # そうでなければ探索点の親へ戻る
            if c == r:
                f = True   
            else:
                c = pl[c]
                
        # 全ての点を探索し終わったか、探索続行不可能で終了
        if k == n or f:
            break
        
    for i in range(n):
        t.vtx[i] = g.vtx[i]
    t.size = k - 1
    t.SetSpringMethodConstants()
    return [t, pl]

# グラフ g を根 r から幅優先探索する
# 返り値は幅優先探索木 t と親のリスト p 
def BFSTree(g, r):
    # t: 幅優先探索木
    # pl: 親のリスト
    # ll: 各 node のレベルを登録するリスト
    # c: 現在の探索点 ( current node )
    # q: 探索点を登録するキュー
    # k: 探索済みの頂点の個数
    
    n = g.ord              # 頂点数
    c = r                   # 探索点の初期値は根 r
    lv = 0                  # レベルの初期値は 0
    t = Graph(n)            # 木の初期化
    pl = [-1] * n           # 親が居ないと -1 を登録
    ll = [-1] * n           # レベル -1 を仮登録
    ll[r] = 0               # 根 r はレベル 0
    q = [r]
    k = 1

    while len(q) > 0 and k < n:
        c = q[0]
        q = q[1:]
        for i in range(n):
            if i != c and g.adj[c][i] > 0 and ll[i] == -1:
            # 現在の探索点の隣接点で未探索の点を探し見つかれば
                pl[i] = c           # 親を登録
                t.adj[i][c] = 1     # 探索木の辺を登録
                t.adj[c][i] = 1
                ll[i] = ll[c] + 1   # レベルを登録
                q.append(i)         # キューに追加
                k += 1

    for i in range(n):
        t.vtx[i] = g.vtx[i]                
    t.size = k - 1
    t.SetSpringMethodConstants()
    return [t, pl]

# 隣接リスト version
def BFSTreeAL(g, r):
    n = g.ord               # 頂点数
    c = r                   # 探索点の初期値は根 r
    lv = 0                  # レベルの初期値は 0
    t = GraphAL(n)          # 木の初期化
    pl = [-1] * n           # 親が居ないと -1 を登録
    ll = [-1] * n           # レベル -1 を仮登録
    ll[r] = 0               # 根 r はレベル 0
    q = [r]
    k = 1
    
    while len(q) > 0 and k < n:
        c = q[0]
        q = q[1:]
        for i in g.adj[c]:
            if i != c and ll[i] == -1:
            # 現在の探索点の隣接点で未探索の点を探し見つかれば
                pl[i] = c           # 親を登録
                t.adj[i].append(c)  # 探索木の辺を登録
                t.adj[c].append(i)
                ll[i] = ll[c] + 1   # レベルを登録
                q.append(i)         # キューに追加
                k += 1

    for i in range(n):
        t.vtx[i] = g.vtx[i]
    t.size = k - 1
    t.SetSpringMethodConstants()
    return [t, pl]

########## グラフ変形関数 ##########

# Graph g の、部分頂点リスト v から誘導される部分グラフ
# ( v はソート済みとする )
def InducedSubGraph(g, v):
    n = len(v)
    h = Graph(n)
    for i in range(n):
        for j in range(n):
            h.adj[i][j] = g.adj[v[i]][v[j]]
    h.size = CountEdges(h)
    h.SetSpringMethodConstants()
    return h

# グラフ g から多重辺とループを取り去り無向化したグラフ
def SimplifiedGraph(g):
    h = GraphCopy(g)
    n = g.ord
    m = 0
    for i in range(n):
        h.adj[i][i] = 0
        for j in range(i + 1, n):
            if g.adj[i][j] > 0 or g.adj[j][i] > 0:
                h.adj[i][j] = 1
                h.adj[j][i] = 1
                m += 1
    h.size = m
    h.dir = 0
    h.SetSpringMethodConstants()
    return h

########## 分解 ##########

# Graph g を連結成分に分解する
# 返り値は連結成分の個数 k とリスト o, v
# o[i] = 第 i 連結成分の頂点数
# v[i] : 第 i 成分の頂点リスト
def ConnectedDecompose(g):
    k = 0
    o = []
    v = []
    n = g.ord
    m = 0               # 探索済みの頂点数
    cl = [False] * n    # 探索済みか否か
    
    while m < n:
        # 未探索点の検索
        r = 0
        while cl[r]:
            r += 1
        # r を根とする広さ優先探索を実行
        t, p = BFSTree(g, r)

        w = []
        for j in range(n):
            # 根 r と、親のいる点は同じ連結成分
            if j == r or p[j] != -1:
                w.append(j)     # 頂点を登録
                cl[j] = True
                m += 1
        o.append(len(w))        # 頂点数を登録
        v.append(w)             # 頂点リストを登録
        k += 1                  # 連結成分数をインクリメント

    return [k, o, v]

"""
/* 単純無向連結グラフ g をブロックに分解する 
   返り値はブロックの個数 k,
   ni=o->mem[i] は第iブロックの頂点数
   si=s->mem[i] は n0,...,n{i-1} の積算
   v->.mem[si],...,v->mem[si+ni-1] は第iブロックの頂点リスト
/* Reference: 
   Chartrand-Oellermann,
   Applied and Algorithmic Graph Theory, Alg.3.2
int BlockDecompose(Graph g, List *o, List *s, List *v)
{
    Graph sc=g; // scan した辺を除去してゆく 
    List pr;    // parent
    List lp;    // low point
    List df;    // depth first search index
    List ss;
    List u;
    int c,r,i=0,j,k=0;

    // BlockDecompose_Step1
    pr=ListInitialize(g.ord,-1);
    df=ListInitialize(g.ord,0);
    o->len=0;
    s->len=0;
    v->len=0;
    ss.len=0;

    BlockDecompose_Step2:
    for(j=0;j<g.ord;j++) if(df.mem[j]==0) break;
    if(j>=g.ord) return k;
    else{
        c=j;    // current vertex
        r=j;    // root
    }
    
    BlockDecompose_Step3:
    i++;
    df.mem[c]=i;
    lp.mem[c]=i;
    ss=Prepend(c,ss);

    BlockDecompose_Step4:
    for(j=0;j<g.ord;j++) if(sc[c][j]) break;

    //BlockDecompose_Step5
    if(j<g.ord){
        sc=RemoveEdge(sc,c,j);
        if(df.mem[j]==0){
            pr.mem[j]=c;
            c=j;
            nextstep BlockDecompose_Step3;
        }
        else{
            // BlockDecompose_Step6
            lp.mem[c]=min(lp.mem[c],df.mem[j]);
            nextstep BlockDecompose_Step4;
        }
    }
    
    // BlockDecompose_Step7
    if(pr.mem[c]!=r){

        // BlockDecompose_Step8
        if(lp.mem[c]<df.mem[pr.mem[c]]){
            lp.mem[pr.mem[c]]=min(lp.mem[c],lp.mem[pr.mem[c]]);
        }
        else{   
            // BlockDecompose_Step9
            // pr.mem[c] is a cut-vertex
            for(j=0;j<ss.len;j++) if(ss.mem[j]==c) break;   
            u=Append(pr.mem[c],SubList(ss,0,j));    // スタックの c までとカット点を抜き出す
            u=Sort(u);
            *s=Append(v->len,*s);       // インデックスを登録
            *v=ListAdd(*v,u);       // 頂点を登録
            *o=Append(u.len,*o);        // 頂点数を登録
            k++;                // ブロック数を更新
            ss=SubList(ss,j+1,ss.len-1);
        }
        
        // BlockDecompose_Step10
        c=pr.mem[c];
        nextstep BlockDecompose_Step4;
    }

    // BlockDecompose_Step11
    for(j=0;j<ss.len;j++) if(ss.mem[j]==c) break;   
    u=Append(r,SubList(ss,0,j));    // スタックの c までと root を抜き出す
    u=Sort(u);
    *s=Append(v->len,*s);       // インデックスを登録
    *v=ListAdd(*v,u);       // 頂点を登録
    *o=Append(u.len,*o);        // 頂点数を登録
    k++;                // ブロック数を更新
    ss=SubList(ss,j+1,ss.len-1);
    
    BlockDecompose_Step12:
    for(j=0;j<g.ord;j++) if(sc[r][j]) break;
    if(j<g.ord){
        c=r;
        nextstep BlockDecompose_Step4;
    }
    else nextstep BlockDecompose_Step2;
}
"""

########## その他 ##########

# u を根とする探索木の、親を登録したリスト pl から u-v 道を探す
def ParentPath(pl, u, v):
    p = [v]         # 道順
    c = v           # 現在位置

    while 1:
        c = pl[c]
        p = [c] + p # 親を順に p に前置きしてゆく
        if c == u:
            break   #u に来たら終了
    return p

# グラフ g で u-v 道を探す 
# u-v 道が存在しなければ返り値は空リスト
def FindPath(g, u, v):
    p = []     # 道順
    t, pl = BFSTree(g, u)
    if pl[v] != -1:
        p = ParentPath(pt, u, v)
    return p

# 全ての頂点の次数が 2 以上の単純連結無向グラフから閉路をひとつみつける
def FindCycle(g):
    h = GraphCopy(g)
    n = g.ord
    p = [0]
    v = [0] * n
    c = 0
    while 1:
        v[c] = 1
        for j in range(n):
            if j != c and h.adj[c][j] > 0:
                h = RemoveEdge(h, c, j)
                c = j
                p.append(c)
                break
        if v[c] > 0:
            break

    j = 0
    while p[j] != c:
        j += 1
    p = p[j:]
    return p

# 頂点数 n のグラフの中の、巡回リスト c で与えられた閉路
def ListtoCycle(n, c):
    g = Graph(n)
    
    for i in range(len(c)):
        g = AddEdge(g, c[i], c[i + 1])
    g.SetSpringMethodConstants()
    return g

# Graph g の次数 1 以上の点のリスト
def PosDegVertices(g):
    s = []
    for i in range(g.ord):
        if CountDegree(g, i) > 0:
            s.append(i)
    return s

# グラフ g の辺 uv が橋であるか否かを判定する
def BridgeQ(g, u, v):
    t, pl = BFSTree(RemoveEdge(g, u, v), u)
    # g から uv を除去したグラフで u を根とする BFS を実行し
    # v に親がいなければ uv は橋
    return (pl[v] == -1)

# グラフ g が連結であるか否かを判定する
def ConnectedQ(g):
    t, pl = BFSTree(g, 0)
    # g で 0 を根とする BFS を実行し
    for i in range(1, g.ord):
        if pl[i] == -1:
            return False
        # 0 以外の或る点に親がいなければ非連結
    return True

# グラフ t が木であるか否かを判定する
def TreeQ(t):
    return (ConnectedQ(t) and CountEdges(t) == (t.ord - 1))

# グラフ g が閉路を含むか否かを判定する
def ExistCycleQ(g):
    # k : 連結成分の個数
    # o : 連結成分の頂点数のリスト
    # v : 連結成分の頂点のリスト

    k, o, v = ConnectedDecompose(g)
    for i in range(k):
        h = InducedSubGraph(g, v[i])    # i 番目の連結成分
        if not TreeQ(h):
            return True
    return False

# 連結重みつきグラフの最小連結子
def MinSpanningTree(g):
    h = GraphCopy(g)
    n = g.ord
    t = Graph(n)
    
    while not TreeQ(SimplifiedGraph(t)):
        m = 0
        for i in range(n):
            for j in range(i + 1, n):
                # 未使用の重み最小の辺 uv を検索
                if h.adj[i][j] > 0:
                    if m == 0:
                        u = i
                        v = j
                        m = h.adj[i][j]
                    else:
                        if h.adj[i][j] < m:
                            u = i
                            v = j
                            m = h.adj[i][j]
                            
        #  閉路ができなければ t に uv を付加
        if BridgeQ(AddEdge(t, u, v), u, v):
            t.adj[u][v] = m
            t.adj[v][u] = m
        
        # h から uv を除去
        h.adj[u][v] = 0
        h.adj[v][u] = 0
        
    t.SetSpringMethodConstants()
    return t

########## オイラー性 ##########

# ランダムなオイラーグラフ
# 頂点数 = n, 辺の割合 = rate
def RandomEulerianGraph(n, rate):
    while 1:
        t = []
        g = RandomGraph(n, rate)
        for u in range(n):
            if VertexDegree(g, u) % 2 == 1:
                t.append(u)
        k = len(t)
        while k > 0:
            i = randint(0, k - 1)
            while 1:
                j = randint(0, k - 1)
                if j != i:
                    break
            u = t[i]
            v = t[j]
            if EdgeQ(g, u, v):
                g = RemoveEdge(g, u, v)
            else:
                g = AddEdge(g, u, v)
            t.remove(u)
            t.remove(v)
            k = len(t)
        if ConnectedQ(g):
            break
    return g

# ランダムな半オイラーグラフ
# 頂点数 = n, 辺の割合 = rate
def RandomSemiEulerianGraph(n, rate):
    g = RandomEulerianGraph(n, rate)
    u = randint(0, n - 1)
    while 1:
        v = randint(0, n - 1)
        if u != v and g.adj[u][v] > 0:
            break
    g = RemoveEdge(g, u, v)
    return g
    
# グラフ g がオイラーグラフ・半オイラーグラフであるか否かの判定
# 返り値は
# オイラーのとき [True]
# オイラーでない半オイラーのとき [True, u, v] （u, v は奇数次数の２点）
# 半オイラーでないとき []
def EulerianGraphQ(g):
    if not ConnectedQ(g):
        return []
        # 連結でなければ偽

    n = g.ord  # 頂点数
    c = 0       # 奇数次数の点の個数
    for i in range(n):
        x = 0   # 次数の計算
        for j in range(n):
            if j != i:
                x += g.adj[i][j]    # 次数の偶奇にループは無関係
        if x % 2 == 1:
            c += 1
            if c == 1:
                u = i
            if c == 2:
                v = i
            if c > 2:
                return []
                # 奇数次数の点が３個以上あれば偽
    if c == 0:
        return [True]
    else:
        return [True, u, v]

# グラフ g が半オイラーグラフであるか否かの判定
# オイラーでない半オイラーのとき真、そうでないとき偽
# 奇数次数の２点も返す
def SemiEulerianGraphQ(g):
    ans = EulerianGraphQ(g)
    if len(ans) == 3:
        return ans
    else:
        return False

# オイラーグラフ g のオイラーサーキット
# Fleury のアルゴリズム
def EulerianCircuit(g):
    n = g.ord          # g の頂点数
    m = CountEdges(g)   # g の辺数
    c = 0               # 現在位置
    h = GraphCopy(g)
    ec = []             # オイラーサーキット
    r = m               # 未使用の辺数
    ec.append(c)
    
    while r > 0:
        t = -1          # 橋による隣接点登録用変数
        b = 0           # 橋以外の辺による隣接点があれば 1
        for i in range(n):
            if h.adj[c][i] > 0:
                if not BridgeQ(h, c, i):
                    # ci が橋でなければ i へ進む
                    h = RemoveEdge(h, c, i)
                    r -= 1
                    ec.append(i) # サーキットに i を登録
                    c = i        # 現在位置を i に更新
                    b = 1        # 橋以外の辺による隣接点があった
                    break   
                else:
                    if t == -1:
                        t = i    # ci が橋のときは t = i を仮登録
                        
        # 橋による隣接点しかないとき
        if b == 0:
            h = RemoveEdge(h, c, t)
            r -= 1
            ec.append(t)    # サーキットに t を登録
            c = t           # 現在位置を t に更新
            
    return ec

# 半オイラーグラフ g の半オイラー歩道を配列で返す
# 返り値は g のサイズ
# 奇数次数の２点 u, v は引数で受け取る
def SemiEulerianWalk(g, u, v):
    ec = EulerianCircuit(AddEdge(g, u, v))
    m = len(ec)
    for i in range(m):
        if (ec[i] == u and ec[i + 1] == v) or (ec[i] == v and ec[i + 1] == u):
            break
    x = ec[0: i]
    y = ec[i + 1:]
    if ec[i] == u:
        x.reverse()
        y.reverse()
        ew = x + y
    else:
        ew = y + x

    return ew

########## 最短路問題 ##########

# 重みつきグラフ g で u から v への最短路 p を求める
# 返り値は最短距離と p
def Dijkstra(g, u, v):
    # tl: 仮ラベル
    # el: 永久ラベル
    # nb: 探索点の隣接点のリスト
    # pt: 親を登録するリスト
    # c : 探索点

    infty = 2 ** 32	# 無限大のつもり
    n = g.ord
    tl = [infty] * n
    el = [-1] * n
    el[u] = 0
    pt = [-1] * n

    c = u
    while 1:
	nb = []
        for i in range(n):
	    # 探索点の未昇格な隣接点を検索
            if el[i] == -1 and g.adj[c][i] > 0:
		nb.append(i)
        k = len(nb)
        for j in range(k):
            i = nb[j]
            x = el[c] + g.adj[c][i]
            if x < tl[i]:
                tl[i] = x    # 仮ラベルの更新
                pt[i] = c    # 仮親を登録

        j = -1
        m = infty
        for i in range(n):
            if el[i] == -1:
                if tl[i] < m:
                    m = tl[i]
                    j = i    
		    # 仮ラベルが正で最小な点 j を検索
        if j >= 0:
            el[j] = tl[j]	# j の仮ラベルを永久ラベルに昇格し
            c = j		# j を次の探索点に
    	if j < 0 or c == v:
	    break

    if c == v:
        p = ParentPath(pt, u, v)
        return [el[v], p]
    else:
        # print "There is no %d-%d path" % (u, v)
        return [0]

########## ネットワークフロー ##########

# ランダムなネットワーク
# 頂点数 = n, 弧の割合 = rate/2, 最大重み = m ( m<=255 に限定 )
# u = 入口、v = 出口、ループはなし
def RandomNetwork(n, u, v, rate, m):
    if m > 255:
        print "最大重みは 255 までにしてください。"
        exit()
        
    while 1:
        g = Graph(n)
        for i in range(n):
            for j in range(n):
                if i != v and i != j and j != u:
                    if random() <= rate:
                        g.adj[i][j] = randint(1, m)
        t, pr = BFSTree(g, u)
        if pr[v] != -1:
            break
    g.SetSpringMethodConstants()
    return g

# ネットワークの最大フロー
# 入り口 = u、出口 = v のネットワーク
# 返り値はフローの値とg の最大フロー
def MaxFlow(g, u, v):
    r = GraphCopy(g)                # 残余フロー
    r.dir = 1
    n = g.ord
    mf = Graph(n)
    t, pr = BFSTree(r, u)
    while pr[v] != -1:
        aug = ParentPath(pr, u, v)  # 増大道を探索
        #WriteGraph(r)
        #print aug
        m = 2 ** 64                 # m = 増大道上の最小重み
        for i in range(1, len(aug)):
            x = r.adj[aug[i-1]][aug[i]]
            if  x < m :
                m = x 
        for i in range(1, len(aug)):
            mf.adj[aug[i - 1]][aug[i]] += m
            r.adj[aug[i - 1]][aug[i]] -= m    
            r.adj[aug[i]][aug[i - 1]] += m
        t, pr = BFSTree(r, u)

    # mf のプラスマイナスを相殺       
    for i in range(n):
        for j in range(i + 1, n):
            if mf.adj[i][j] > mf.adj[j][i]:
                mf.adj[i][j] -= mf.adj[j][i]
                mf.adj[j][i] = 0
            else:
                mf.adj[j][i] -= mf.adj[i][j]
                mf.adj[i][j] = 0
                
    # フローの値の計算
    vl = 0
    for i in range(n):
        if i != u:
            vl += mf.adj[u][i]
    return [vl, mf]

# 二部グラフの最大マッチング
# 無向グラフのままでは正しく動かない
def MaxMatchingBipartite(g):
    m = len(g.v1)
    n = len(g.v2)
    p = m + n
    h = GraphCopy(g)
    h.dir = 1
    # 有向化し第２群から第１群への弧を除去
    for i in range(m, p):
        for j in range(m):
            h.adj[i][j] = 0
    h = AddVertexDirected(h, g.v1, [])
    # 頂点 p を付加し、頂点の第１群へ隣接させる
    h = AddVertexDirected(h, [], g.v2)
    # 頂点 p + 1 を付加し、頂点の第２群から隣接させる
    vl, mf = MaxFlow(h, p, p + 1)
    # 付加した２頂点間の最大フローを求め
    m = SimplifiedGraph(RemoveVertex(RemoveVertex(mf, p + 1), p))
    # 付加した２頂点を除去すれば良い
    return [vl, m]

# 二部グラフ G 内のマッチング M に関する r を根とする交代木
# 返り値は交代木、親のリスト、増大道
def AlternatingTree(g, mu, r):
    # t: 交代木
    # pl: 親のリスト
    # cl: 探索が終わったか否かを登録するリスト ( closed list )
    # k: 探索済みの頂点の個数
    # c: 現在の探索点 ( current node )
    # b: 探索木をバックトラック ( backtrack ) するか否か
    # f: 連結成分を探索し終わったか否か ( finish )
    # mu: マッチングの相手を登録したリスト
    #     ( mu[i] = i <=> i is a single vertex )

    n = g.ord               # 頂点数
    c = r                   # 探索点の初期値は根 r
    t = Graph(n)            # 木の初期化
    pl = [-1] * n           # 親が居ないと -1 を登録
    cl = [False] * n
    cl[r] = True            # 根 r は探索済み
    k = 1
    level = 0

    augpathfound = False
    while True:

        # 偶数レベルでの作業
        if level % 2 == 0:
            b = True
            f = False
            for i in range(n):
                # 現在の探索点の隣接点で未探索の点を探し見つかれば木を伸ばす
                if i != c and g.adj[c][i] > 0 and not cl[i]:    
                    pl[i] = c       # 親を登録
                    t.adj[i][c] = 1     # 探索木の辺を登録
                    t.adj[c][i] = 1 
                    cl[i] = True    # 探索済みリストに登録
                    k += 1
                    c = i           # 探索点を i に進める
                    b = False       # バックトラックは不要
                    level += 1
                    
                    # i が single vertex なら M-増大道ができるのでここで終了
                    if mu[i] == i:
                        augpathfound = True

                    break
                
            # バックトラックが必要なとき
            if b:
                # 現在点が r ならこれ以上の探索は不可能
                # そうでなければ探索点の親の親へ戻る
                if c == r:
                    f = True   
                else:
                    c = pl[pl[c]]
                    level -= 2
                    
        # 奇数レベルでの作業
        else:
            i = mu[c]
            pl[i] = c       # 親を登録
            t.adj[i][c] = 1     # 探索木の辺を登録
            t.adj[c][i] = 1 
            cl[i] = True    # 探索済みリストに登録
            k += 1
            c = i           # 探索点を i に進める
            b = False       # バックトラックは不要
            level += 1
                    
        # M-増大道がみつかったか、全ての点を探索し終わったか、探索続行不可能で終了
        if augpathfound or k == n or f:
            break
        
    for i in range(n):
        t.vtx[i] = g.vtx[i]
    t.size = k - 1
    t.SetSpringMethodConstants()

    if augpathfound:
        aug = ParentPath(pl, r, c)
    else:
        aug = []
    return [t, pl, aug]


# 重み付き二部グラフの最大マッチング
# Kuhn-Munkres
def MaxMatchingWeightedBipartite(g):

    # 頂点数が不揃いのときは頂点と重さ 0 の辺を追加
    # 交代木探索のため全ての辺に重み 1 を加算
    m = len(g.v1)
    n = len(g.v2)
    if m > n:
        p = m
    else:
        p = n
    q = 2 * p   # = h.ord    
    h = RandomBipartiteGraph(p, p, 0)
    h.wt = 1
    for i in range(m):
        for j in range(n):
            h.adj[i][p + j] = g.adj[i][m + j]
            h.adj[p + j][i] = g.adj[m + j][i]
    for i in range(p):
        for j in range(p, q):
            h.adj[i][j] += 1
            h.adj[j][i] += 1

    # feasible vertex labeling
    fvl = [0] * q
    for i in range(p):
        x = 0
        for j in range(p, q):
            if h.adj[j][i] > x:
                x = h.adj[j][i]
        fvl[i] = x
    for j in range(p, q):
        fvl[j] = 0

    def constructGl():
        k = RandomBipartiteGraph(p, p, 0)
        s = 0
        for i in range(p):
            for j in range(p, q):
                if fvl[i] + fvl[j] == h.adj[i][j]:
                    k.adj[i][j] = 1
                    k.adj[j][i] = 1
                    s += 1
        k.size = s
        return k
    
    gl = constructGl()
    [vl, match] = MaxMatchingBipartite(gl)
    
    # mu: マッチングの相手を登録したリスト
    # mu[i] = i <=> i is a single vertex
    mu = range(q)
    for i in range(p):
        for j in range(p, q):
            if match.adj[i][j] > 0:
                mu[i] = j
                mu[j] = i
                break
                    
    nextstep = 31
    finish = False
    while not finish:

        if 0:
            print "H:"
            WriteGraph(h)
            print "fvl for V1:", fvl[:p]
            print "fvl for V2:", fvl[p:]
            print "G_l:"
            WriteGraph(gl)
            print "M:"
            WriteGraph(match)

        if nextstep == 31:
            finish = True
            for i in range(p):
                if VertexDegree(match, i) == 0:
                    finish = False
                    break
            if not finish:
                nextstep = 32
            else:
                break

        if nextstep == 32:
            x = i
            nextstep = 33

        if nextstep == 33:            
            [t, pl, aug ] = AlternatingTree(gl, mu, x)
            #print "x =", x
            #print "T:"
            #WriteGraph(t)
            #print "Aug. Path:", aug
            # 増大道がみつかっていればマッチングを増大
            if len(aug) > 0:
                #print "augment"
                for i in range(len(aug) - 1):
                    u = aug[i]
                    v = aug[i + 1]
                    if i % 2 == 0:
                        match.adj[u][v] = 1
                        match.adj[v][u] = 1
                        mu[u] = v
                        mu[v] = u
                    else:
                        match.adj[u][v] = 0
                        match.adj[v][u] = 0
                        
                nextstep = 31
            else:
                nextstep = 4

        if nextstep == 4:

            if t.size == 0:
                vt = [x]
            else:
                vt = []
                for i in range(q):
                    if VertexDegree(t, i) > 0:
                        vt.append(i)
            #print "V(T) =", vt
            w = 2 ** 64
            for i in range(p):
                if i in vt:
                    for j in range(p, q):
                        if not j in vt:
                            y = fvl[i] + fvl[j] - h.adj[i][j]
                            if y < w:
                                w = y
            #print "w =", w
            for i in range(p):
                if i in vt:
                    fvl[i] -= w
            for j in range(p, q):
                if j in vt:
                    fvl[j] += w
            nextstep = 5

        if nextstep == 5:
            gl = constructGl()
            nextstep = 33

    k = GraphCopy(g)
    vl = 0
    for i in range(m):
        for j in range(n):
            if match.adj[i][p + j] == 0:
                k.adj[i][m + j] = 0
                k.adj[m + j][i] = 0
            else:
                vl += k.adj[i][m + j]

    return [vl, k]

# 最大マッチング
# Edmonds' Blossom-Shrinking Algorithm
# Korte-Vygen, Combinatorial Optimization, Chapter 10
#
def MaxMatching(g):
    n   = g.ord
    mu  = range(n)
    phi = range(n)
    rho = range(n)
    scanned = [False] * n

    def p(v):
        a = [v]
        x = v
        b = 0
        while 1:
            if b == 0:
                x = mu[x]
            else:
                x = phi[x]
            if x in a:
                break
            else:
                a.append(x)
                b = 1 - b
        return a

    def rootQ(x):
        return (mu[x] == x)

    # oddptQ  is not  ( not evenptQ )
    def oddptQ(x):
        w = mu[x]
        return (w != x and phi[w] == w and phi[x] != x)

    def evenptQ(x):
        w = mu[x]
        return (w == x or phi[w] != w)
    
    def outofforestQ(x):
        w = mu[x]
        return (w != x and phi[w] == w and phi[x] == x)    

    nextstep = 2
    while 1:
        if nextstep == 2:
            finish = True
            for x in range(n):
                if evenptQ(x):
                    if not scanned[x]:
                        finish = False
                        break
            if finish:
                break
            else:
                nextstep = 3

        if nextstep == 3:
            found = False
            for y in range(n):
                if g.adj[x][y] > 0:
                    if outofforestQ(y) or (rho[y] != rho[x] and (evenptQ(y))):
                        found = True
                        break
            if found:
                nextstep = 4
            else:
                scanned[x] = True
                nextstep = 2

        # grow
        if nextstep == 4:
            if outofforestQ(y):
                #print "grow"
                phi[y] = x
                nextstep = 3
            else:
                nextstep = 5

        # augment
        if nextstep == 5:
            a = p(x)
            b = p(y)
            #print "P(x):", a
            #print "P(y):", b
            c = ListCommon(a, b)
            if len(c) == 0:
                #print "augment"
                for i in range(1, len(a), 2):
                    v = a[i]
                    mu[phi[v]] = v
                    mu[v] = phi[v]
                for i in range(1, len(b), 2):
                    v = b[i]
                    mu[phi[v]] = v
                    mu[v] = phi[v]
                mu[x] = y
                mu[y] = x
                for v in range(n):
                    phi[v] = v
                    rho[v] = v
                    scanned[v] = False
                nextstep = 2
            else:
                nextstep = 6

        # shrink
        if nextstep == 6:
            #print "shrink"
            for i in range(len(c)):
                r = c[i]
                if rho[r] == r:
                    break
            pxr = a[0: a.index(r) + 1]
            for i in range(1, len(pxr), 2):
                v = pxr[i]
                if rho[phi[v]] != r:
                    phi[phi[v]] = v
            pyr = b[0: b.index(r) + 1]
            for i in range(1, len(pyr), 2):
                v = pyr[i]
                if rho[phi[v]] != r:
                    phi[phi[v]] = v
            if rho[x] != r:
                phi[x] = y
            if rho[y] != r:
                phi[y] = x
            u = ListUnion(pxr, pyr)
            for v in range(n):
                if rho[v] in u:
                    rho[v] = r
            nextstep = 3

        if 0:
            print "mu :", mu
            print "phi:", phi
            print "rho:", rho
            print "scanned:", scanned
            print

    m = Graph(n)
    for x in range(n):
        y = mu[x]
        if y != x:
            m.adj[x][y] = 1
            m.adj[y][x] = 1
    m.size = CountEdges(m)
    return [m.size, m]
                
"""     
# 重み付きマッチング
# Edmonds' algorithm
# Korte-Vygen, Combinatorial Optimization, Chapter 10
#
def WeightedMatching(g):
    infty = 2 ** 64
    n = g.ord
    
    #1 initialize
    b = [[v] for v in range(n)]     # blossoms
    k = 0
    delta = 0
    z = []
    for v in range(n):        
        x = infty
        for w in range(n):
            if g.adj[v][w] < x:
                x = g.adj[v][w]
        z.append(x / 2.0)
    zeta = ListCopy(z)
    k   = [0] * n
    mu  = range(n)
    rho = [[0] * n for i in range(n)]
    phi = [[0] * n for i in range(n)]
    for i in range(n):
        rho[0][i] = i
    for i in range(n):
        phi[0][i] = i
    mark = [0] * n  # 0: even / 1: odd / 2: out of forest

    tA = [0] * n
    tAv = [[0] * n for i in range(n)]
    txv = [[0] * n for i in range(n)]
    tau = [[0] * n for i in range(n)]
    tauxv = [[0] * n for i in range(n)]
    
    def maxblossomQ(i):
        ans = True
        for j in range(len(b)):
            if j != i:
                if not ListIncludedQ(s, t):
                    ans = False
                    break
        return ans

    def evenblossomQ(a):
        if mark[a[0]] == 0:
            return True
        else:
            return False

    def oddblossomQ(a):
        if mark[a[0]] == 1:
            return True
        else:
            return False

    def outofforestblossomQ(a):
        if mark[a[0]] == 2:
            return True
        else:
            return False

    def slack(v, w):
        if g.adj[v][w] == 0:            return infty
        else:
            x = 0
            for i in range(len(b)):
                a = b[i]
                if ((v in a) and (not w in a)) or ((w in a) and (not v in a)):
                    x += z[i]
            return g.adj[v][w] - x

    def update(v):
        for x in range(n):
            tauxv[x] = x
            txv[x] = g.adj[v][x] - zeta[v] + delta
        for i in range(len(b)):
            
        return    

    nextstep = 2
    while 1:
        if nextstep == 2:
            t = [infty] * len(b)
            for v in range(n):
                if mark[v] == 0:
                    update(v)u
                
"""   
        

